;;=========================================
;; File: app.clp             Feb 23rd, 1998
;; 
;; Author: Jessica Chen-Burger
;;=========================================

(clear)
(unwatch all)

; This is the top level of all other files for BSDM DB
; It should be called as "hardy -clips app.clp &"

; Naming the application

(hardy-set-title 
   "Knowledge Based Support Tool for Enterprise Modelling by Jessica Chen-Burger")
(hardy-set-author "(c) Jessica Chen-Burger")
(hardy-set-name "KBST-EM")
(hardy-set-about-string 
     "A Knowledge-Based Support Tool for Enterprise Modelling")

;; pc version 
(hardy-set-help-file "hardy.hlp")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;; load hardy-html translation programs
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; Globals
(load "globals-html.clp")

;;; HtmlWriter class declaration
(load "htmlgen-html.clp")

;;; Generates an image map for a diagram
(load "genmap-html.clp")

;;; Dialog box for entering pathname, etc.
(load "dialog-html.clp")

;; Diagram card conversion
(load "diagram-html.clp")

;; Hypertext card conversion
(load "hypertxt-html.clp")

;; Text card conversion
(load "text-html.clp")

;; Main conversion routines
(load "convert-html.clp")

;; Install a menu callback
(register-event-handler CustomMenu Toplevel trans-custom-menu)

;; Construct an HtmlWriter instance
(bind ?*html-writer* (make-instance html-writer of HtmlWriter))

;;=========================================
;; Shared Functions 
;;=========================================

;; global variables
(load "global.clp")

(load "library.clp")
(load "comm.clp")
(load "dialog.clp")

;;=========================================
;; Functions for leftclick
;;=========================================

;;; rep and broadcast racd nodes
(load br-racd.clp)

;;; chk racd node attributes consistency
(load chk-racd.clp)

(load "left-all.clp")

(register-event-handler NodeLeftClick "Ontology" all-left-click)
(register-event-handler NodeLeftClick "Activity Evolution Map"
                                      all-left-click)

;;=========================================
;; Functions for CustomMenu Button
;;=========================================

;;; search any node, called by leftclick/custom
(load c-seek.clp)

(load "idef3.clp") ;; not used at the moment 

(load "metabsdm.clp")   ;; trans from meta-mode to bsdm
(load "metauml1.clp")   ;; trans from meta-model to UML Class dia
(load "uml1meta.clp")   ;; trans from UML Class dia to meta-model
(load "racdmm.clp")   ;; trans from RACD to MM 

(load "objcon.clp") ;; change obj name and maintain consistency
                             ;; in a modelling language

(load "meta.clp")       ;; modelling in meta
(load "repall.clp")     ;; rep bsdm and meta models
(load "broadcast.clp")  ;; broadcast atts

(load "quick.clp")      ;; rep all nodes (only racd
                                 ;; implemented so far)

(load "bsdm.clp")       ;; bsdm local card functions


(load "edify.clp")        ;; rep all edify cards 
(load "edify-t.clp")      ;; edify - transform activity to 
                          ;; system calls 

(load "custom.clp")

(register-event-handler CustomMenu "Ontology" bsdmem-custom-menu)
(register-event-handler CustomMenu "Activity Evolution Map" 
                                                  bsdmem-custom-menu)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; On creating nodes 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(load "new-node.clp")


(register-event-handler CreateNodeImage "Ontology" event-new-node)
(register-event-handler CreateNodeImage "Activity Evolution Map"
                                                     event-new-node)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; End of File
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

