;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; File: br-racd.clp
;; 
;; Broadcast one racd node info within only racd: 
;; i.e. activity to an activity, and role to a role
;;
;; rep all important nodes on all RACD cards. 
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;     (load ./system/br-racd.clp)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; used by quick.clp 
(defglobal ?*all-racd-node-names* = -1)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; rep all atts of the selected node
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(deffunction rep-racd-atts (?card ?obj ?type ?name)

   (bind ?att (diagram-object-get-first-attribute 
               ?card ?obj))

   (while (neq ?att "")
     (bind ?value (diagram-object-get-string-attribute 
                   ?card ?obj ?att))

     (if (and (neq ?att type) 
              (neq ?att "type") 
              (neq ?att "id") 
              (neq ?att "card") 
              (neq ?att "images") 
              (neq ?att "arcs") ) then 
     (assert (racd_att ?card ?type ?name ?att ?value))  )

     (bind ?att (diagram-object-get-next-attribute))
   ) ;; while
) ;; deffun 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; represent all nodes info on ONE RACD card
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(deffunction rep-racd-card (?card)

  ;;; get all images ;;; 
  (bind ?image (diagram-card-get-first-node-image ?card))
  (while (neq ?image -1) 

    ;; get node details
    (bind ?obj  (diagram-image-get-object ?card ?image))
    (bind ?type (diagram-object-get-string-attribute ?card ?obj type))
    (bind ?name (diagram-object-get-string-attribute ?card ?obj "Name"))

    ;; only rep the following nodes, and will be used by this file 
    ;; and quick.clp 
    (if (or (eq ?type "Activity") 
            (eq ?type "Role") 
            (eq ?type "Data") 
            (eq ?type "Data Store") 
            (eq ?type "Collective Activity") 
        )  then 

        (assert (racd_card ?card node_info ?type ?name ?image ?obj))
        (rep-racd-atts ?card ?obj ?type ?name)

        ;; store node name - used by quick.clp 
        (if (eq (member$ ?name ?*all-racd-node-names*) FALSE) then 
                (bind ?*all-racd-node-names* 
                (create$ ?*all-racd-node-names* ?name)) );; if
    );; if 

    ;; get next node id
    (bind ?image (diagram-card-get-next-node-image))
  );; while

  (return 1)
) ;; deffun 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Top Function (1)
;; 
;; rep all RACD cards 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(deffunction rep-racd-top ()

  (bind ?ans (message-box 
        "Do you want to represent RACD nodes - 
this will CLEAR your previous representation.
You also need to close all menu windows."  YES-NO))
  (if (neq ?ans YES) then 
      (return -1) )

  (unwatch all)  (reset)

  ;; will be used by quick.clp 
  (bind ?*all-racd-node-names* (create$))

  ;; get card id
  (bind ?card (hardy-get-first-card))
  (while (neq ?card -1)

      ;; get card type
      (bind ?card-type (my-card-get-type2 ?card))
 
      (if (eq ?card-type "RACD")  then 
          (card-show ?card) (card-iconize ?card)

          ;; represent RACD model nodes info
          (rep-racd-card ?card)

          (card-quit ?card)      
      );; if 

      ;; next card 
      (bind ?card (hardy-get-next-card))
  );; while

  ;; used by quick.clp 
  (bind ?*all-racd-node-names* (string-sort ?*all-racd-node-names*))

  (message-box "Finished Representing RACD Models." OK)
  (return 1)
  (watch fact)
) ;; deffun

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; broadcast rules 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defrule broadcast-racd-node-1
  (broadcast transfer racd_node ?type ?name ?card ?image ?obj)
  (racd_card ?card2 node_info ?type ?name ?image2&~?image ?obj2)
=>
  ;; copy everything, including empty field
  (copy-all-field ?card ?obj ?card2 ?obj2))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defrule broadcast-racd-node-2
  (broadcast overwrite racd_node ?type ?name ?card ?image ?obj)
  (racd_card ?card2 node_info ?type ?name ?image2&~?image ?obj2)
=>
  ;; copy everything, including empty field
  (overwrite-all-field ?card ?obj ?card2 ?obj2))
;;;;;;;;;;;;;;;;;;;;;;;;
(defrule clear-broadcast-racd-node
  (declare (salience -10000))
  ?d <- (broadcast racd_node $?)
=>
  (retract ?d)   )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Top Function (2) Broad RACD Node - 
;; activity and role
;;
;; Two options are available: transfer or overwrite 
;; transfer copies only non-empty attributes !! 
;; overwrite copies everything !! 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(deffunction broadcast-racd-top (?card ?image ?option)

  ;; get node details
  (bind ?obj  (diagram-image-get-object ?card ?image))
  (bind ?type (diagram-object-get-string-attribute ?card ?obj type))
  (bind ?name (diagram-object-get-string-attribute ?card ?obj "Name"))
 
  (if (and (neq ?type "Activity")(neq ?type "Role")
           (neq ?type "Data")    (neq ?type "Data Store") 
           (neq ?type "Collective Activity") 
       ) then
   (message-box "Please select a (Collective) Activity/Role/Data/Data Store." OK)
   (return -1)  );;if 

  (assert (broadcast ?option racd_node ?type ?name ?card ?image ?obj))
  (run)

  (message-box "Finished broadcast." OK)
  (return 1)
);; deffun

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; TOP FUNCTION (0)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(deffunction broadcast-racd-atts (?card ?image ?node-type ?node-name) 

      ;; simple if the model has been represented 
      (if (eq ?*all-racd-node-names* -1) then 
          (rep-racd-top) )

      (bind ?ans (message-box "Do you want to overwrite other nodes?
If so, choose YES. Otherwise, choose NO then only 
the non-empty attributes are transfered !! (YES)" YES-NO))
 
      ;; "transfer" copies only non-empty attributes !! 
      ;; "overwrite" copies everything !! 

      (if (eq ?ans YES) then 
          (broadcast-racd-top ?card ?image overwrite) );; if

      (if (eq ?ans NO) then 
          (broadcast-racd-top ?card ?image transfer) );; if
      (return 0)
); deffun

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; end of file 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
