;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; File: broadcast.clp 
;;
;; works on meta-model and BSDM only. 
;;
;; Functions: 
;;
;; 1. broadcast EVERY class in the meta-model to all entities
;;    in the BSDM model
;;
;; 2. broadcast a meta-class within the meta-model only. 
;; 
;; uses: repall.clp metabsdm.clp 
;;    (load "./system/broadcast.clp")
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; PART I
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Rule (1.1) 
;; 
;;     using rep in  repall.clp 
;;
;;     (assert (meta_model ?card class "Abstract Class" ?name ?obj))
;;     (assert (meta_model ?card class "Concrete Class" ?name ?obj))
;;     (assert (bsdm_card  ?card entity ?name ?obj))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;    (matches   broadcast-meta-bsdm-rule)

(defrule broadcast-meta-bsdm-rule
  (broadcast meta_bsdm)
  (meta_model ?meta-card class  ? ?ent-name ?class-obj)
  (bsdm_card  ?bsdm-card entity   ?ent-name ?ent-obj)
  (not (updated-temp ?bsdm-card ?ent-name ?ent-obj))
=>
  (card-show ?meta-card)
  (card-show ?bsdm-card)

  ;; calling metabsdm.clp 
  (mapping-to-bsdm-entity
   ?meta-card ?class-obj ?bsdm-card ?ent-obj )

  (assert (updated-temp ?bsdm-card ?ent-name ?ent-obj))
  (card-quit ?meta-card)
  (card-quit ?bsdm-card)
) ;; defrule

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Rule (1.-) Clean UP
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defrule broadcast-clean-up
   (declare (salience -10000))
   (or ?d <- (broadcast $?)
       ?d <- (updated-temp $?) )
=>
   (retract ?d)  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Top Function (1) 
;;  
;; callling: repall.clp
;;
;;    (broadcast-meta-bsdm)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(deffunction broadcast-meta-bsdm ()

   (rep-all-models) 
   (assert (broadcast meta_bsdm))
   (run)

   (message-box "Finished Broadcast All Meta Classes to BSDM." OK)
   (return 1)
) ;; deffun 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; PART II
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; used global variables
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; (defglobal ?*select-card*  = -1)
;; (defglobal ?*select-image*  = -1)
;; (defglobal ?*select-obj*  = -1)
;; (defglobal ?*select-node-name*  = -1)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; rules 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; used facts
;;  (meta_model ?card class "Concrete Class" ?name ?obj)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;    (matches  broadcast-meta-only-rule )

(defrule broadcast-meta-only-rule
  (broadcast meta_only ?card  ?obj   ?name ?type)
  (meta_model          ?card2 class  ?type ?name ?obj2)
=>
  ;; the test of neq or <> somehow does not work properly
  ;; when it is on the left hand side.

  (if (<> ?obj ?obj2) then 
  (overwrite-all-field     ?card ?obj ?card2 ?obj2 ))
) ;; defrule

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Top Function (2) 
;;  
;; callling: repall.clp
;;
;; ensure consistency within meta model
;; calling from custom menu - after selecting the class
;; 
;; finish debugging
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;   (load "./system/broadcast.clp")
;;
;;   (broadcast-meta-only 7994)
;;   (assert (broadcast meta_only ?*select-card* ?*select-obj* 
;;                      ?*select-node-name* "Concrete Class" ))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(deffunction broadcast-meta-only (?card)

   (bind ?obj   ?*select-obj*)
   (bind ?name  ?*select-node-name*)

   (if (or (neq ?card ?*select-card*)
           (eq ?*select-obj*  -1)) then 
       (message-box "Please select a meta-class." OK) 
       (return -1) )       

   (bind ?type (diagram-object-get-string-attribute ?*select-card*
                ?*select-obj* type ))
   
   (printout t ?*select-card* crlf)
   (printout t ?*select-obj* crlf)
   (printout t ?*select-node-name* crlf)
   (printout t ?type crlf)

   (if (and (neq ?type "Concrete Class")
            (neq ?type "Abstract Class")) then 
        (message-box "Please select a concrete or abstract class." OK)
        (return -1) )

   (unwatch all) ;; to speed up the program 
   (rep-meta-model)
   
   (watch facts)
   (assert (broadcast meta_only ?card ?obj ?name ?type))
   (run)

   (message-box "Finished Broadcasting." OK)
   (return 1)
) ;; deffun 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; end of file 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
