;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; File: chk-racd.clp
;;
;; calling br-racd.clp
;; called  by  left-all.clp
;; 
;; check the att consistency in a racd card.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;    (load ./system/chk-racd.clp)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; inconsistency window design
;; 
;; called when inconsistency in the att is found.
;; simple-menu is in comm.clp
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(deffunction    att-inconsistency-window-design 
  (?type ?name ?att-name ?card1 ?value1 ?card2 ?value2)

  ;; if error card then return -1
  (if (or (< ?card1 0) (< ?card2 0)) then 
      (message-box "Error: card found in att-inconsistency-window" OK)
      (return -1) )

  (bind ?title-1 (card-get-string-attribute ?card1 title))
  (bind ?title-2 (card-get-string-attribute ?card2 title))

  ;;;;;;;;;;;;;;;;;;; Create Frame ;;;;;;;;;;;;;;;;;;;;;;;
  (bind ?frame (frame-create 0 "Inconsistency Report" -1 -1 500 500))
  (window-add-callback ?frame OnClose on-close)
  (window-add-callback ?frame OnMenuCommand simple-menu)

  ;;;;;;;;;;;; Menu & Manu bars ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

  ;;;;; create a menu ;;;;;;; NOT USED 
  ;;(bind ?edit-menu (menu-create))
  ;; add menu item, id = 1
  ;;(menu-append ?edit-menu 1 "&Modify")

  ;;;;; create a menu ;;;;;;
  (bind ?exit-menu (menu-create))
  ;; add item id=9 
  (menu-append ?exit-menu 9 "&Cancel")

  ;;;;;;;;;;;;;;; Create Menu Bar ;;;;;;;;;;;;;
  (bind ?menu-bar (menu-bar-create))

  ;;; add Menues to this Menu Bar
  ;; (menu-bar-append ?menu-bar ?edit-menu "&Modify")
  (menu-bar-append ?menu-bar ?exit-menu "&Cancel")

  ;;; add the menu-bar to the frame !!
  (frame-set-menu-bar ?frame ?menu-bar)

  ;;;;;;;;;; Create a panel on frame ;;;;;;;;;
  (bind ?panel (panel-create ?frame 0 0 280 250))
  (panel-set-label-position ?panel wxVERTICAL)

  ;;; create text (entity name) on the panel, and new line
  (message-create ?panel ?name)
  (panel-new-line ?panel)

  (bind ?msg (str-cat "Type: " ?type))
  (message-create ?panel ?msg)
  (panel-new-line ?panel)

  (bind ?msg (str-cat "Attribute Name: " ?att-name))
  (message-create ?panel ?msg)
  (panel-new-line ?panel)

  (message-create ?panel "Found Attribute Values From Diagrams:")
  (panel-new-line ?panel)

  ;;;;;;;; display  att values ;;;;;;;;;

  (bind ?panel-value (multi-text-create 
        ?panel "" ?title-1 ?value1 -1 -1 500 200))
  (panel-new-line ?panel)

  (bind ?panel-value (multi-text-create 
        ?panel "" ?title-2 ?value2 -1 -1 500 200))
  (panel-new-line ?panel)

  ;;;;;;;; fit the content to window ;;;;;;;;;
  (window-fit ?panel)
  (window-fit ?frame)

  ;;;;; show window ;;;;;
  (window-show ?frame 1)

  (return 1)
);; deffun
 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Rules 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; collect error atts in a list
;;
;; (matches collect-error-att-list)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defrule collect-error-att-list
   (chk-racd-atts   ?type ?name)
   (racd_att ?card1 ?type ?name ?att ?value1)
   (racd_att ?card2 ?type ?name ?att ?value2&~?value1)
   (not (racd_att_list_temp ?type ?name ?att ?card1 ?value1 ?card2 ?value2))
   (not (racd_att_list_temp ?type ?name ?att ?card2 ?value2 ?card1 ?value1))
=> 
   (assert (racd_att_list_temp ?type ?name ?att ?card1 ?value1 ?card2 ?value2))
);; defrule 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; rule: report error atts 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defrule report-inconsistent-atts
   (chk-racd-atts      ?type ?name)
   (racd_att_list_temp ?type ?name ?att ?card1 ?value1 ?card2 ?value2)
   (not (att-inconsistency-reported ?type ?name ?att 
                       ?card1 ?value1 ?card2 ?value2))
=> 
   (att-inconsistency-window-design 
           ?type ?name ?att ?card1 ?value1 ?card2 ?value2)

   (assert (att-inconsistency-reported ?type ?name ?att 
                       ?card1 ?value1 ?card2 ?value2))
) ;; defrule 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defrule  report-inconsistent-atts-clear
   (declare (salience -10000))
   ?d  <-   (chk-racd-atts $?)
   ?d1 <-   (racd_att_list_temp $?)
=> 
   (retract ?d ?d1) )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Top Function 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(deffunction check-racd-att-consistency 
             (?obj-type ?name)

  ;; a simple if the model has been represented 
  (if (eq ?*all-racd-node-names* -1) then 
      (rep-racd-top) )

   (assert (chk-racd-atts ?obj-type ?name))
   (run)
   (message-box "Finish Attribute Checking." OK)
);; deffun 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; end of file 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
