;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; File: custom.clp 
;;
;; Author: Jessica Chen-Burger
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;; main function for custom menu ;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;            
;;   (load ./system/custom.clp)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;            

(deffunction bsdmem-custom-menu (?card ?menu-item) 

    (bind ?card-type (my-card-get-type2 ?card))

    (printout t "========calling custom-menu========" crlf) 
    (printout t "card type: " )
    (printout t ?card-type crlf) 

     ;;;;;;;;;; common functions ;;;;;;;;;;;

    (if (eq ?menu-item "Browse Cards") then
        (open-card-catalogue)
        (return 1)  )

    (if (eq ?menu-item "Close All Cards") then
        (close-all-cards close-card)
        (return 1) )

    (if (eq ?menu-item "Iconize All Cards") then
        (close-all-cards iconize-card)
        (return 1) )

    (if (eq ?menu-item "Show File Name") then
        (get-file-name ?card)(return 1) )

    (if (eq ?menu-item "Show Card Type") then
        (get-type ?card)(return 1) )

    (if (eq ?menu-item "Uniform All Names") then
        (uniform-names ?card)(return 1) )

    (if (eq ?menu-item "Update Attributes") then
        (update-attributes ?card)(return 1) )

    (if (eq ?menu-item "Save All Diagrams") then
        (save-all-diagrams)(return 1)  )

    (if (eq ?menu-item "Search Node") then
        (search-node-top)(return 1)  )

    ;;;;;;;;;; RACD Operations ;;;;;;;;;;;
   
    (if (eq ?menu-item "Show All RACD Nodes") then
        (show-all-racd-nodes-top)(return 1)  )

    (if (eq ?menu-item "Delete Shadow") then
        (my-delete-shadow) (return 1)  )

    ;;;;;;;;;; IDEF3 Operations ;;;;;;;;;;;

    (if (eq ?menu-item "Create IDEF3 Card") then
        (create-idef3-card ?card)(return 1) )

    ;;;;;;;;;;; copy functions ;;;;;;;;;;;

    (if (eq ?menu-item "Abstract to Concrete") then 
        (copy-meta-class ?card "concrete")
        (return 1) )

    (if (eq ?menu-item "Concrete to Abstract") then 
        (copy-meta-class ?card "abstract")
        (return 1) )

    ;;;;;;;;;;; translation functions ;;;;;;;;;;;

    (if (eq ?menu-item "Meta-Class to BSDM") then 
        (meta-to-bsdm ?card "entity")
        (return 1) )

    (if (eq ?menu-item "Meta-Class to UML Class") then 
        (meta-to-uml-1 ?card "UML Class")
        (return 1) )

    (if (eq ?menu-item "UML Class to Meta-Class") then 
        (uml-1-to-meta ?card  "Meta Class")
        (return 1) )

    ;; RACD node to a MM concrete class 
    (if (eq ?menu-item "RACD To MM") then 
        (racd-to-mm ?card) 
        (return 1) )

    ;;;;;;;; Change Object Name: Local Consistency ;;;;;;;;;

    (if (eq ?menu-item "Change Object Name") then 
        (bind ?ans (message-box 
"The option enables you to change the NAME of the 
selected modelling object within the selected 
modelling lagnuage. Would you like to proceed?" YES-NO))
        (if (eq ?ans YES) then 
            (global-change-object-name ?card) 
        );; if 
        (return 1)
     );; if

    ;;;;;;;;;;; Edify Card Functions ;;;;;;;;;;;
    ;; Edify 

    (if (eq ?menu-item "Export Edify Activities") then
        (rep-all-edify-cards)
        (return 1) )

    (if (and (eq ?card-type "Edify") 
             (eq ?menu-item "Transform to System Call"))
         then
         (transform-to-system-call ?*select-card* ?*select-image*)
        (return 1) )

    ;;;;;;;;;;; Broadcast Functions ;;;;;;;;;;;
    ;; meta-model menu

    ;; This function can be extended from MM to all other models. 
    (if (eq ?menu-item "Broadcast To All Models") then 
        (message-box "Only implemented from MM to BSDM." OK)
        (broadcast-meta-bsdm)
        (return 1) )

    ;; meta-model 
    (if (eq ?menu-item "Broadcast Within MM") then 
        (broadcast-meta-only ?card)
        (return 1) )

    ;; RACD diagram
    (if (eq ?menu-item "Broadcast To MM") then 
        (message-box "Under Construction." OK)
        (return 1) ) 

    ;; RACD diagram
    (if (eq ?menu-item "Broadcast In RACD") then 
        (message-box "Please CTRL+Left-click on Node." OK)
        (return 1) )

);; end all-custom-menu
