;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; File: dialog.clp
;; 
;; Author: Jessica Chen-Burger
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;     (load "./system/dialog.clp")
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defglobal ?*old-obj-name-slot* = -1)
(defglobal ?*new-obj-name-slot* = -1)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(deffunction execute-change-object-name-1 
             (?card  ?target-obj-type 
              ?field-name ?old-name ?new-name)

  (printout t "--- execute change object name-1 ---" crlf)
  (printout t ?target-obj-type "=> " ?field-name "=> "
              ?old-name "=> " ?new-name crlf)

  (bind ?ans NO) 
  (if (< ?card 0) then (return -1) )
  (card-show ?card)

  ;; get image id
  (bind ?image (diagram-card-get-first-node-image ?card))
  (while (> ?image 0) 

    ;; get node details
    (bind ?obj  (diagram-image-get-object ?card ?image))
    (bind ?type (diagram-object-get-string-attribute ?card ?obj type))
    (bind ?name (diagram-object-get-string-attribute ?card ?obj ?field-name))

    ;; change object name 
    (if (and (eq ?type ?target-obj-type)
             (eq ?name ?old-name) )   then 

     ;; note: the selected image can not be shadowed 
     ;; at the same time, but other images can be shown shadowed
     (diagram-image-set-shadow-mode ?card ?image 1)

     (bind ?msg (str-cat "Do you want to change the
selected/shadowed node field name <" ?field-name "> 
from <" ?old-name "> 
to <" ?new-name ">"  ))

     (bind ?ans (message-box ?msg YES-NO))
     (if (eq ?ans YES) then 
         (diagram-object-set-string-attribute 
          ?card ?obj ?field-name ?new-name)  );; if 

     (my-delete-shadow-2 ?card ?image)
     (my-show-node-name-2 ?card ?image)

    );; if-change object name

    ;; get next image id
    (bind ?image (diagram-card-get-next-node-image))
  ) ;; while

  ;; if ?ans = NO, then this card has not been modified
  (if (eq ?ans NO) then (card-quit ?card) )
  (return 1) 
) ;; deffun 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(deffunction execute-change-object-name 
     (?target-card-type ?target-obj-type 
      ?field-name ?old-name ?new-name)

    (printout t "--- execute change object name ---" crlf)

    ;; library.clp 
    (bind ?card-catalogue (get-all-cards))

    (bind ?first-card-name (nth$ 1 ?card-catalogue))
    (bind ?first-card (card-find-by-title ?first-card-name))

    (while (> ?first-card 0)

      (bind ?first-card-type (my-card-get-type2 ?first-card))

      (if (eq ?target-card-type ?first-card-type) then 
          (execute-change-object-name-1 
                     ?first-card  ?target-obj-type 
                     ?field-name ?old-name ?new-name)
      );; if 

      (bind ?card-catalogue  (rest$  ?card-catalogue))
      (bind ?first-card-name (nth$ 1 ?card-catalogue))
      (bind ?first-card (card-find-by-title ?first-card-name))
    ) ;; while 

    (return 1)
) ;; deffun 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(deffunction dialog-box-menu (?frame ?id)

 ;;;;;; (1) run/action ;;;;; 
 (if (eq ?id 1) then

    (if (or (< ?*old-obj-name-slot* 0)
            (< ?*new-obj-name-slot* 0)) then 
        (message-box "Please quite the window and re-do again." OK)
        (return -1)   ) 

    (bind ?old-name (text-get-value ?*old-obj-name-slot*))
    (bind ?new-name (text-get-value ?*new-obj-name-slot*))

    (printout t ?old-name crlf)
    (printout t ?new-name crlf)

    (if (or (eq ?old-name "")(eq ?new-name "")) then 
        (message-box "Error: Please give both old and new name." OK)
        (return -1) );; if 

    (execute-change-object-name 
          ?*select-card-type* ?*select-obj-type*
          "Name" ?old-name ?new-name)

     (message-box "Action finished." OK)
     (return 1)
 ) ; end-id-2

 ;;;;;; (2) cancel frame ;;;;;;
 (if (eq ?id 9) then
     (window-delete ?frame)  
     (return 1)
 ) ; end-id-2

 (return 0)
) ;; deffun 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Main Function
;; 
;; (my-dialog-box-create "diagram-card-type" "obj-type"  "old-name") 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(deffunction my-dialog-box-create (?card-type ?obj-type ?old-name)

  ;;;;;;;;;;; create a frame/window ;;;;;;;;;;;;;;;;;
  (bind ?att-frame (frame-create 0 "Model Object Consistency Menu"
                     -1 -1 300 300))

  ;;; the callback fun is stored in att.clp
  (window-add-callback ?att-frame OnClose on-close)
  (window-add-callback ?att-frame OnMenuCommand dialog-box-menu)

  ;;;;;;;;;;;;; Create Menu-buttons and Menu Bar ;;;;;;;;;;;;;;;

  ;;; create a menu item (1), run ;;; 
  (bind ?run-menu (menu-create))
  (menu-append ?run-menu 1 "&Run")

  ;;; create a menu item (9), exit  ;;; 
  (bind ?exit-menu (menu-create))
  (menu-append ?exit-menu 9 "&Quit")

  ;;;;;;;;;; create menu bar ;;;;;;;;;;
  (bind ?att-menu-bar (menu-bar-create))

  ;;; add Menu item to this Menu Bar
  (menu-bar-append ?att-menu-bar ?run-menu "&Run")

  ;;; add Menu items to this Menu Bar
  (menu-bar-append ?att-menu-bar ?exit-menu "&Quit")

  ;;; add the menu-bar to the frame 
  (frame-set-menu-bar ?att-frame ?att-menu-bar)

  ;;;;;;;;;;;; Create a panel on att-frame ;;;;;;;;;;;;;;;;;;

  (bind ?panel (panel-create ?att-frame 0 0 300 300))
  (panel-set-label-position ?panel wxHORIZONTAL)

  ;;;;; create panel items (1) ;;;;;
  (message-create ?panel "           Global Update Of A Model Object")
  (panel-new-line ?panel)
  (panel-new-line ?panel)

  (message-create ?panel "Diagram Card Type:")
  (message-create ?panel ?card-type)
  (panel-new-line ?panel)

  (message-create ?panel "Object Type:")
  (message-create ?panel ?obj-type)
  (panel-new-line ?panel)

  ;;;;; create panel items (2) ;;;;;
  (bind ?*old-obj-name-slot* (text-create ?panel "" 
                   "Old Object Name" ?old-name -1 -1 300))
  (panel-new-line ?panel)

  ;;;;; create panel items (3) ;;;;;
  (bind ?*new-obj-name-slot* (text-create ?panel "" 
                       "New Object Name" "" -1 -1 300))
  (panel-new-line ?panel)

  ;;;;;;;; fit the content to window ;;;;;;;;;
  (window-fit ?panel)
  (window-fit ?att-frame)

  ;;;;; show window ;;;;;
  (window-show ?att-frame 1)

  (return 1)
) ;; deffun 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; end of file 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
