;;=======================================================
;; File: library.clp                  May 29th, 1998
;; Author: Jessica Chen-Burger
;;
;; diagram-object-set-string-attribute is the same as
;; set-object-string-attribute (the latter one is an older
;; version)
;; 
;; Note: a system call has been made, therefore, this file
;; is only running on SUN only.
;; This file stores functions called by custom-menu
;;---------------------------------------------------------
;;    (load ./system/library.clp)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defglobal ?*select-card*  = -1)
(defglobal ?*select-image*  = -1)
(defglobal ?*select-obj*  = -1)
(defglobal ?*select-node-name*  = -1)
;;=======================================================
;;                Library Functions
;;=======================================================
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Delete the double/multiple spaces between two words:
;; 
;; (delete-multi-spaces "create originate      focal entity")
;; => "create originate focal entity"
;; (delete-multi-spaces "create originate focal entity")
;; => "create originate focal entity"
;; (delete-multi-spaces "   create originate focal     entity  ")
;; => "create originate focal entity"
;; (delete-multi-spaces "") => ""
;; (delete-multi-spaces "      ") => " "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(deffunction delete-multi-spaces (?string)
  (if (eq ?string "") then (return ""))
   (bind ?pos (str-index "  " ?string))
   (bind ?len (str-length ?string))
   ;; delete double spaces in the string
   (while (and (> ?len 0) (neq ?pos FALSE))
     (bind ?string (str-cat (sub-string 1 (- ?pos 1) ?string)
                            (sub-string (+ ?pos 1) ?len ?string)))
     (bind ?pos (str-index "  " ?string)) 
     (bind ?len (str-length ?string))
   )
   ;;;  delete a space at the front, if any.
   (bind ?pos (str-index " " ?string))
   (if (and (> ?len 0)(eq ?pos 1)) then 
          (bind ?string (sub-string 2 ?len ?string))  )

   ;;;  delete a space at the end, if any.
   (bind ?len (str-length ?string))
   (if (eq " " (sub-string ?len ?len ?string)) then 
       (bind ?string (sub-string 1 (- ?len 1) ?string))  )

   (return ?string)   );; deffun
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; (my-stru-to-string "['xxx', 'yyy', 'zzz']" )
;; Output = "'xxx', 'yyy', 'zzz'"
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(deffunction my-stru-to-string (?structure)
   (if (or (eq ?structure "") (eq ?structure "[ ]"))
       then (return FALSE) )
   (bind ?len (str-length ?structure))
   (sub-string 2 (- ?len 1) ?structure)   )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; return the first item in the ?structure
;; (my-first "'xxx', 'yyy', 'zzz'" )
;; Output = "'xxx'"
;; (my-first "xxx, yyy, zzz" )
;; Output = "xxx"
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(deffunction my-first (?structure)
   ;; no item left in the list
   (if (eq ?structure "") then (return "") )

   ;; last item in the list
   (bind ?pos (str-index "," ?structure))
   (if (eq ?pos FALSE) then (return ?structure))

   ;; more than one items in the list
   (return (sub-string 1 (- ?pos 1) ?structure)) 
)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; It delete everything before the first comma, and the it 
;; skips all the spaces after this comma and return the 
;; rest of the list. 
;; e.g.
;; (my-rest "  " )         => ""
;; (my-rest " zzz  " )     => ""
;; (my-rest "'xxx',    'yyy',   'zzz'" )
;; => "'yyy', 'zzz'"
;; (my-rest "   xxx,   yyy,   zzz   " )
;; => "yyy, zzz"
;; (str-index , "")    => FALSE
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(deffunction my-rest (?structure)
   ;; no item left in the list
   (if (eq ?structure "") then (return ""))

   ;; last item in the list
   (bind ?pos (str-index "," ?structure))
   (if (eq ?pos FALSE) then (return ""))

   ;; more than one items in the list
   (bind ?length (str-length ?structure))
   (bind ?structure (sub-string (+ ?pos 1) ?length ?structure))

   (if (neq ?structure "") then 
       (bind ?structure (delete-multi-spaces ?structure))
       (return ?structure)
    else  (return ""))     )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; found if ?item is in the list of ?structure
;; (my-member "'xx x'"    "['xx x', 'yyyy', 'zzzz', 'kkk']")
;; => TRUE
;; (my-member "'Theme Type'" "['Duration', 'Theme Type']")
;; => TRUE
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(deffunction my-member (?item ?structure)

   ;; no item in the structure
   (if (eq ?structure "") then (return FALSE))

   (bind ?length (str-length ?structure))
   (bind ?structure (sub-string 2 (- ?length 1) ?structure))
   (bind ?pos (str-index ?item ?structure))

   (if (eq ?pos FALSE) then (return FALSE) )
   (return TRUE)
)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; (my-list-to-multi "['xxx   sss',    'yyy', 'zzz']"  )
;; =>   ("'xxx sss'" "'yyy'" "'zzz'"), a multiple field value
;; (my-list-to-multi "['Person', 'Practical Allocation']")
;; => ("'Person'" "'Practical Allocation'")
;; (my-first "'Person', 'Practical Allocation'")   
;;  (my-first "") => "", and  (my-rest "") => ""
;;  (my-first "'aaa'") => "'aaa'", and  (my-rest "'aaa'") => ""
;; (my-list-to-multi "['xxx']"  )
;; => ("'xxx'")
;; (my-list-to-multi "['Module', 'Person']")
;; => ("'Module'" "'Person'")
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(deffunction my-list-to-multi (?structure)
   ;; no item in the structure
   (if (eq ?structure "") then (return ""))
   (bind ?length (str-length ?structure))

   ;;;;;;; take out the bracket, [] ;;;;;
   (bind ?structure (sub-string 2 (- ?length 1) ?structure))
   (bind ?structure (delete-multi-spaces ?structure))
   (bind ?length (str-length ?structure))

   (bind ?first     (my-first ?structure))
   (bind ?res (create$ ))
   (while (and (neq ?first "")(neq ?structure "") )
       (bind ?first (delete-multi-spaces ?first))
       (bind ?res (create$ ?res ?first))
       (bind ?structure (my-rest ?structure))  
       (bind ?first     (my-first ?structure))
   );; while
   (return ?res)   )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; ?structure = "['xxx', 'yyy', 'zzz']"
;; output = the number of elements in the ?structure, e.g. 3 in
;; the above case
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(deffunction my-length   (?structure)
   (if (eq ?structure "") then (return 0))
   (if (eq ?structure "[ ]") then (return 0))
   (bind ?multi_field (my-list-to-multi   ?structure))
   (return (length$ ?multi_field))
)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; ?end = "xxxx"   ?structure = "'ddd ddd ddd'"  
;; output = TRIE, if ?end is the ending of ?structure;
;;          otherwise, FALSE.
;;
;; (my-string-ending "s" "'Registration Forms'") => true
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(deffunction my-string-ending (?end ?structure)
    (if (eq ?structure "") then (return FALSE))
    (if (eq ?end "") then (return TRUE))

    (bind ?pos (str-index ?end ?structure))
    (bind ?len (- (length$ ?structure) 1))

    (while (and (neq ?pos FALSE) (neq ?structure ""))

      (bind ?structure (sub-string ?pos ?len ?structure))
      (if (eq ?end ?structure) then (return TRUE))

      (bind ?structure (sub-string 2 ?len ?structure))
      (bind ?pos (str-index ?end ?structure))
      (bind ?len (length$ ?structure))
    )
    (return FALSE) )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; ?item = "'sss'", ?structure = "'xxxxxx'"  
;; output = TRUE, if ?item is a part of ?structure;
;;          otherwise, FALSE.
;; 
;; e.g.  CLIPS> (my-substring "'business'" "'create business'")
;;       CLIPS> TRUE
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(deffunction my-substring (?item ?structure)
    (if (or (eq ?structure "") (eq ?item ""))
        then (return FALSE))

    (bind ?len (str-length ?structure))
    (bind ?structure (sub-string 2 (- ?len 1) ?structure))
    (bind ?len (str-length ?item))
    (bind ?item (sub-string 2 (- ?len 1) ?item))

    (bind ?pos (str-index ?item ?structure))
    (if (eq ?pos FALSE) then (return FALSE) 
     else  (return TRUE))     );deffun
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Input: ?structure1, ?structure2 = "[ ]"
;;        ?structure1, ?structure2 = "['xxx', 'yyy']"
;; Output = TRUE, if content of ?structure1 = ?structure2
;;          Otherwise, FALSE.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(deffunction my-eq (?structure1 ?structure2)
  (if (eq ?structure1 ?structure2) then (return TRUE))

  (bind ?len1 (str-length ?structure1))
  (bind ?len2 (str-length ?structure2))
  (if (neq ?len1 ?len2) then (return FALSE))

  (bind ?multi-1 (my-list-to-multi ?structure1))
  (bind ?multi-2 (my-list-to-multi ?structure2))
  (if (and (subsetp ?multi-1 ?multi-2)
           (subsetp ?multi-2 ?multi-1))
   then (return TRUE)
   else (return FALSE) ) )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; return TRUE, if multi-field-1 is the same
;; set as the multi-field-2
;; (equal-set (create$ 1 5 2 6 4) (create$ 5 2 1 4 6))
;; => TRUE
;; (equal-set (create$ 1 5 2 6) (create$ 5 2 1 4 6))
;; => FALSE
;; (equal-set (create$ 1 2 3 1 ) (create$ 3 1 2 ))
;; => 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(deffunction equal-set (?multi-1 ?multi-2)
  (if (eq ?multi-1 ?multi-2) then (return TRUE))
  (if (and (subsetp ?multi-1 ?multi-2)
           (subsetp ?multi-2 ?multi-1)) 
   then    (return TRUE)
   else    (return FALSE))    ) ;; deffun

;;====================================================================
;;                   library function
;;====================================================================
;;              my-db-get (?index  ?database)
;;
;; (my-db-get address (create$ john surename Mac address))
;; ""
;; (my-db-get address (create$ john surename Mac address 15))
;; 15
;; (my-db-get add (create$ john surename Mac address 15))
;; ""
;; (my-db-get add (create$))
;; ""
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(deffunction my-db-get (?index ?database)

   (bind ?pos (member$ ?index ?database))
   (if (eq ?pos FALSE) then (return ""))

   (bind ?res (nth$ (+ ?pos 1) ?database))
   (if (eq ?res nil) then (return ""))
   (return ?res)     )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;      my-db-replace (?index ?new-value ?database)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; (my-db-replace  add 15/8 (create$ add 15))
;; (add 15/8)
;;
;; (my-db-replace  add 15/8 (create$ nothing inside))
;; (nothing inside)
;;
;; (my-db-replace  add 15/8 (create$))
;; ()   
;;
;; (my-db-replace  add "" (create$ add 15))
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(deffunction my-db-replace (?index ?new-value ?database)
   (bind ?pos (member$ ?index ?database))
   (if (eq ?pos FALSE) then (return ?database))

   (bind ?database 
        (replace$ ?database (+ ?pos 1) (+ ?pos 1) ?new-value))
   (return ?database)
) ;; deffun 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                  my-del-multi
;;
;;  (my-del-multi  3 (create$ 1 2 3 3))
;;  (1 2 3)
;;  (my-del-multi  4 (create$ 1 2 3 3))
;;  (1 2 3 3)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(deffunction my-del-multi (?ele ?multi)

   (bind ?pos (member$ ?ele ?multi))
   (if (eq ?pos FALSE) then (return ?multi))

   (bind ?res (delete$ ?multi ?pos ?pos))
   (return ?res)   ) ;; deffun 

;;=======================================================

;; this function is not in use yet !! 
;; copy all images from ?from-card to ?to-card
;; (copy-card 21095 21811)

(deffunction copy-card (?from-card ?to-card)
   (card-select-all    ?from-card)
   (diagram-card-copy  ?from-card)
   (diagram-card-paste ?to-card)  )
;;---------------------------------------------------------
;; to close/iconize all opened cards
;;  (close-all-cards iconize-card)
;;  (close-all-cards close-card)

(deffunction close-all-cards (?action)
   
    (bind ?card (hardy-get-first-card))
    (while (> ?card 0)

      (card-iconize ?card)
      (card-iconize ?card 1)

      (if (eq ?action close-card) then 
          ;; notice card-quit doesn't work with options.
          (card-quit ?card) )

      (bind ?card (hardy-get-next-card))
    );; while
    (return 1)  );; deffun
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; return true, if two input strings are equal
;; return false, if not equal.
;; (my-equal-string " this is JUST a test   " "This   is Just a test" )
;; => TRUE
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(deffunction my-equal-string (?string1 ?string2)
   
   (bind ?string1 (delete-multi-spaces ?string1))
   (bind ?string2 (delete-multi-spaces ?string2))

   ;; 1. To convert two string into two lower case strings
   (bind ?string1 (lowcase ?string1))
   (bind ?string2 (lowcase ?string2))

   (if (eq ?string1 ?string2) then 
       (return TRUE) else
       (return FALSE) )  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; to refresh a diagram card !!

(deffunction my-refresh-diagram (?card)
  (card-show                 ?card)
  (diagram-card-clear-canvas ?card)
  (diagram-card-redraw       ?card) )

;;------------------------------------------------------------
;; to show the top window - browser frame - therefore to 
;; envisible several dialog boxes.

(deffunction show-browser-frame ()
 (bind ?f (hardy-get-browser-frame))
 (window-show ?f)
) ;deffun
;;---------------------------------------------------------
;; Save all to persistent database ;;

;; Call-back function. Save current working data to 
;; card.db, before the user quit kbst. 

(deffunction quit-kbst ()

    ;; remember to check all un-saved def-win-2
    ;; windows (att.clp)

    (assert (chk-save-before-quit def-win-2))
    (run)

    ;; mainly to delete att.clp's frame-info
    (assert (del-frame-info everything))
    (run)

;; Save information to a persistent data file: card.db.
;; i.e. the error messages: predicate (error-open-file)

;;    (message-box "System messages are saved in card.db." OK)


      (if (eq (get-platform) "Motif") then
          ;; sun version
          (system "cp card.db card.bak")
       else
          ;; pc version
         (system "copy card.db card.bak")
      )

      (save-facts "card.db")
)

;;---------------------------------------------------------
;; mainly to delete att.clp's and prodef.clp's
;; frame-info

(defrule del-frame-info-everything
   (del-frame-info everything)
   (or ?d <-  (att-list-menu $?)
       ?d <-  (att-form-info  $?)
       ?d <-  (del-att-from-object $?)
       ?d <-  (prodef-menu-frame-info $?)
       ?d <-  (pro-scope-frame-info $?)
       ?d <-  (att-def-form $?)
   )
 =>  
  (retract ?d) )
;;---------------------------------------------------------
; the last cleaning command

(defrule last-clean-all
   (declare (salience -10000))
   (or ?d <- (del-frame-info everything)
       ?d <- (chk-save-before-quit def-win-2)
   )
  =>  
   (retract ?d) )
;;---------------------------------------------------------
;; Get file name of a card  

(deffunction get-file-name (?card)

  (bind ?file-name (card-get-string-attribute ?card "filename"))
  (if (eq  "" ?file-name) then 
          (message-box "No file name found." OK)
   else   (message-box (str-cat "File Name: " ?file-name) OK) )
); deffun

;;---------------------------------------------------------
;; Get type of a card 

(deffunction get-type (?card)

  (bind ?type (card-get-string-attribute ?card "diagram-type"))

  (if (eq  "" ?type) then 
      (bind ?type (card-get-string-attribute ?card "type")) )

  (if (eq  "" ?type) then 
          (message-box "Unable to find type." OK) )

  (message-box (str-cat "Card ID: " ?card ", Card Type: " ?type) OK)
); deffun


;;---------------------------------------------------------
;; Bring all entity to the front on a BSDM card ;;

;; note that the type "entity" is spelled with lower case in ER
;; modelling, and is spelled with upper case "Entity" in BSDM.

(deffunction bring-all-entities-to-front (?card)
   
  (if (and (neq "bsdm" (card-get-string-attribute ?card "diagram-type"))
           (neq "Dynamic Business Model" (card-get-string-attribute 
                        ?card "diagram-type"))  )
      then  (return -1) )

    (card-set-status-text ?card "Bring entities to front")

  ;;;;;;;; bring entities to front ;;;;;;;;;;;
  ;;get the first image as current image
  (bind ?image (diagram-card-get-first-node-image ?card))

  ;; while the current image is not -1 then do...
  (while (neq ?image -1) 

  (bind ?object (diagram-image-get-object ?card ?image))
  (bind ?type   (diagram-object-get-string-attribute ?card ?object "type"))

    ;; (1) An entity is before process, so put to the front
    (if (or (eq ?type "Entity") 
            (eq ?type "Entity Occurrence") )   then
        (diagram-image-put-to-front ?card ?image 1)  )

    ;; (2) if the image is a process then put to back (0)
    (if (or (eq ?type "Process") 
            (eq ?type "Process Type") )   then
        (diagram-image-put-to-front ?card ?image 0)  )

    ;; get the next image, repeat loop
    (bind ?image (diagram-card-get-next-node-image))
    ) ;; while

  ;;;;;;;; bring postit to front ;;;;;;;;;;;
  ;;get the first image as current image
  (bind ?image (diagram-card-get-first-node-image ?card))

  ;; while the current image is not -1 then do...
  (while (neq ?image -1) 

  (bind ?object (diagram-image-get-object ?card ?image))
  (bind ?type   (diagram-object-get-string-attribute ?card ?object "type"))

    ;; (2) if the image is a Postit, yellow-sticker, 
    ;;     then put to front (1)
    (if (eq ?type "Postit") then
        (printout t ?type crlf)
        (diagram-image-put-to-front ?card ?image 1) )

  ;; get the next image, repeat loop
  (bind ?image (diagram-card-get-next-node-image))
  ) ;; while

    ;;;;;;;;;;;;;;;;; done the work ;;;;;;;;;;;;;;
    (my-refresh-diagram ?card)
    (card-set-status-text ?card "Finish bring entities to front.")
    (return 1)
);; deffun

;;==========================================================
;; Get all cards in the application  ;;
;; return a string which contains all the card titles
;;==========================================================
(deffunction get-all-cards ()

    (bind ?card (get-first-card))
    (if (and (neq ?card -1)(= 1 (card-is-valid ?card)) ) then
        (bind ?card-catalogue (get-card-string-attribute ?card title))
    )

    (bind ?card (get-next-card))
    (while (and (neq ?card -1) (= 1 (card-is-valid ?card)) )
      (bind ?card-title (get-card-string-attribute ?card title))
      (bind ?card-catalogue (mv-append ?card-catalogue ?card-title))
      (bind ?card (get-next-card))       ); while

    (if (eq ?card-catalogue "") then (return -1)
     else 
        (bind ?card-catalogue (string-sort ?card-catalogue))
        (return ?card-catalogue)
    )  )

;;==========================================================
;; Custom function 
;; 
;; Retrieve all cards and open a view menu 
;; retrieve all cards' name in an ascending order
;;==========================================================
(deffunction open-card-catalogue ()

  (bind ?card-catalogue "")
  (bind ?card-catalogue (get-all-cards)) 

  ;;;;; open the view window using choice window ;;;;;
  (bind ?choice (get-choice "Card Catalogue" ?card-catalogue))
  (if (neq ?choice "") then 
      (bind ?view-id (find-card-by-title ?choice))
      (card-show ?view-id)
      (return 1)
  )
  (return -1)   ); deffun 
;;==========================================================
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Common Functions
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; get diagram card type !
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(deffunction my-card-get-type2 (?card)

  (bind ?type (card-get-string-attribute ?card "diagram-type"))

  (if (eq "" ?type) then
      (bind ?type (card-get-string-attribute ?card "type")) )

  ;; it is proved that all RACD diagram cards has shown the 
  ;; correct "RACE" diagram type. 
  ;; (printout t ?type crlf) 

  (if (eq "" ?type) then
      (message-box "ERROR: Unable to find card type." OK)
      (return -1) )

  (return ?type)
); deffun
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;      (save-all-diagrams)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(deffunction save-all-diagrams ()

  ;; save index, not used, don't know file name
  ;; (hardy-save-index ?file)

  ;; get card id
  (bind ?card (hardy-get-first-card))
  (while (> ?card 0)

     ;; tested, not need to open a card to save a diagram
     ;; also, only the updated daigrams will be saved
     ;; (card-show ?card)

     (bind ?type (my-card-get-type2 ?card))
     (bind ?file (card-get-string-attribute ?card filename))
     (bind ?modified (card-is-modified ?card ))

     (printout t "-- save card --" crlf)
     (printout t ?type crlf)
     (printout t ?file crlf)

     ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
     (bind ?ans YES) 
     (if (eq ?file "") then 
     (card-show ?card 1) 
     (bind ?ans (message-box 
           "File Name is missing. Continue?" YES-NO))
     (if (eq ?ans NO) then (return -1))   ) ;; if 
     ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
     (bind ?ans YES) 
     (if (eq ?type "") then 
     (card-show ?card 1) 
     (bind ?ans (message-box 
           "Diagram Type is missing. Continue?" YES-NO))
     (if (eq ?ans NO) then (return -1))   ) ;; if 
     ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
     (if (eq ?modified 1) then 
     ;; (1) save diagram cards
     (if (eq ?type "Diagram card") then 
         (diagram-card-save-file ?card ?file) )

     ;; (2) save hypertext cards
     (if (eq ?type "Hypertext card") then 
         (hypertext-card-save-file ?card ?file) )

     ;; (3) save text cards, not used, don't know ?window
     ;; (if (eq ?type "Text card") then 
     ;;     (text-window-save-file ?window ?file) )
     )  ;; if-modified 

     (card-quit  ?card)
     (bind ?card (hardy-get-next-card))
  );; while

  (message-box "All Diagrams Cards Have Been Saved." OK)
  (return 1)
);; deffun 
;;==========================================================
;; select the node, then delete the shadow
;;
;;   (my-delete-shadow)
;;==========================================================
(deffunction my-delete-shadow ()

  (bind ?card  ?*select-card* )
  (bind ?image ?*select-image*)

  (diagram-image-set-shadow-mode ?card ?image 0)
  (my-refresh-diagram ?card)    )
;;==========================================================
(deffunction my-delete-shadow-2 (?card ?image)

  (diagram-image-set-shadow-mode ?card ?image 0)
  (my-refresh-diagram ?card)    )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(deffunction my-show-node-name (?card ?obj) 

 (diagram-object-format-text ?card ?obj) 

 ;; need to clear the canvas after redraw so to clear
 ;; leftover old stuff.

 (diagram-card-redraw ?card) 
 (diagram-card-clear-canvas  ?card)
 (diagram-card-redraw ?card) 

 (return 1)

) ;; deffun 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(deffunction my-show-node-name-2 (?card ?image) 

 (bind ?obj (diagram-image-get-object ?card ?image))

 (diagram-object-format-text ?card ?obj) 

 ;; need to clear the canvas after redraw so to clear
 ;; leftover old stuff.

 (diagram-card-redraw ?card) 
 (diagram-card-clear-canvas  ?card)
 (diagram-card-redraw ?card) 

 (return 1)

) ;; deffun 

;;---------------------------------------------------------
;; Purpose: Delete all extra spaces in a string and then 
;; convert all the letters into lower cases.
;; (my-convert-string "  this is  a  test  ")
;; =>  "This Is A Test"

(deffunction my-convert-string (?string)

   (bind ?string (delete-multi-spaces ?string))
   (bind ?string (lowcase ?string))
   (bind ?res "")

   (while (neq ?string "") 
    ; convert the first letter into capital in the string
    (bind ?len (str-length ?string))
    (bind ?first-letter (upcase (sub-string 1 1 ?string)))
    (bind ?string (str-cat ?first-letter (sub-string 2 ?len ?string)))

    ; find first space and cut it out of the string
    (bind ?space (str-index " " ?string))
    (if (eq ?space FALSE) then 
        (bind ?res (str-cat ?res ?string))
        (bind ?string "")
     else
        (bind ?res (str-cat ?res (sub-string 1 ?space ?string)))
        (bind ?string (sub-string (+ ?space 1) ?len ?string)) )
   )     
   (return ?res)  

);; deffun 

;;---------------------------------------------------------
;; Convert all object name starting with a capital letter

(deffunction uniform-names (?card)

     (printout t "calling uniform-names.... " ) 
     (printout t ?card crlf)

     (bind ?ans (get-text-from-user 
           "Please specify the type of nodes, or just click ok
- Activity Instance, Association Instance are already included." ))

     ;; get first image
     (bind ?image (diagram-card-get-first-node-image ?card ))

     (while (> ?image 0)
        (bind ?obj (diagram-image-get-object ?card ?image))
        (bind ?type (diagram-object-get-string-attribute ?card ?obj type))
        (printout t ?type crlf) 

        (if (or (eq ?type "Activity Instance")
                (eq ?type "Association Instance" )
                (eq ?type ?ans) )
         then 
          (bind ?string 
                (diagram-object-get-string-attribute ?card ?obj "Name" ))
          (printout t ?string ) 
          (printout t "-->" )
          (bind ?new (my-convert-string ?string ))
          (printout t ?new crlf)

          ;; update image name
          (diagram-object-set-string-attribute ?card ?obj "Name" ?new)
          (diagram-object-format-text ?card ?obj) 
        ) ;; if 
        
        ;; get next image
        (bind ?image (diagram-card-get-next-node-image ))
      ) ;; while 

     ;; refresh card
     (my-refresh-diagram ?card)

) ;; deffun

;; -----------------------------------------------------------

;; ?card = card-ID 
;; ?att = a string, the attribute name
;; ?new-value = a string, the new value to replace the old value

(deffunction update-attributes (?card) 

    (printout t "calling fill-att-value...." crlf)

    ;; get attribute and new-value
    (bind ?att (get-text-from-user "Please specify the attribute name." ))

    (bind ?msg (str-cat "Please specify the new attribute value for attribute: " ?att ))
    (bind ?new-value (get-text-from-user ?msg))

    (if (eq ?att "") then (return -1))

    ;; find first image
    (bind ?image (diagram-card-get-first-node-image ?card ))

    (while (> ?image 0)

        ;; test image type
        (bind ?obj   (diagram-image-get-object ?card ?image))
        (bind ?name (diagram-object-get-string-attribute ?card ?obj "Name"))
        (bind ?value (diagram-object-get-string-attribute ?card ?obj ?att))
        
        (bind ?msg (str-cat " The Node: " ?name "
has attribute: " ?att " 
has value: " ?value ".
Would you like to replace it with: " ?new-value " ?" ))

        (bind ?ans (message-box ?msg YES-NO ))

        ;; update image
        (if (eq ?ans YES) then 
            (diagram-object-set-string-attribute ?card ?obj ?att ?new-value ))

        ;; find next image
        (bind ?image (diagram-card-get-next-node-image ))

    ) ;; while 

) ;; deff

;;==========================================================
;;                 end of file
;;==========================================================
