;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; File:  metabsdm.clp, from Meta-Model to BSDM
;; 
;;  Author: Jessica Chen-Burger
;;  (load "/hame/jessicac/kbst-em/system/metabsdm.clp")
;; 
;; Function: translate a meta-model object to a BSDM entity
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Use the below global variables
; (defglobal ?*select-card*  = -1)
; (defglobal ?*select-image*  = -1)
; (defglobal ?*select-obj*  = -1)
; (defglobal ?*select-node-name*  = -1)
; (defglobal ?*meta-bsdm* =  -1) 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; mapping the fields in the meta-mode to 
;; fields in a bsdm entity 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(deffunction  mapping-to-bsdm-entity
   (?from-card ?from-obj ?to-card ?to-obj)

   (printout t "mapping-to-bsdm-entity" crlf)

   ;;;;;;; mapping same att names ;;;;;;
   
   (copy-all-field
    ?from-card ?from-obj ?to-card ?to-obj)

   ;;;;;;; mapping different names ;;;;; 
   (bind ?mapping-info ?*meta-bsdm*)

   (bind ?from-field   (nth$ 1 ?mapping-info))
   (bind ?to-field     (nth$ 2 ?mapping-info))
   (bind ?mapping-info (rest$ (rest$ ?mapping-info)))

   (while (and (neq ?from-field nil) 
               (neq ?to-field   nil) )

       ;; calling comm.clp 
       (instantiate-field ?from-card ?from-obj ?to-card ?to-obj
                          ?from-field ?to-field)

     (bind ?from-field   (nth$ 1 ?mapping-info))
     (bind ?to-field     (nth$ 2 ?mapping-info))
     (bind ?mapping-info (rest$ (rest$ ?mapping-info)))
   ) ;; while

   ;; to show entity name
   (my-show-node-name ?to-card ?to-obj) 

) ;; deffun 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; top level function (1) 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; create an entity and then copy all 
;; attributes in the abstract/concrete class to entity 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(deffunction meta-to-bsdm (?card ?entity)

   (bind ?card-type (card-get-string-attribute ?card diagram-type))

   (if (neq ?card-type "bsdm") then 
       (return -1) )

   (bind ?image (node-image-create ?card "Entity"))
   (bind ?obj (diagram-image-get-object ?card ?image))
   (diagram-image-move ?card ?image 50 50)

   ;; translate the meta-model class to bsdm entity 
   (if (eq ?entity "entity") then 
       (mapping-to-bsdm-entity
          ?*select-card* ?*select-obj* ?card ?obj))

   ;; translate the meta-model class to bsdm process

) ;; deffun 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
