;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; File:  metauml1.clp, from Meta-Model to UML class model
;; 
;;  Author: Jessica Chen-Burger
;;  (load "/hame/jessicac/kbst-em/system/metauml1.clp")
;; 
;; Function: translate a meta-model class to a UML class
;; Status:   working fine 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Use the below global variables to rep selected node: 
; 
; (defglobal ?*select-card*  = -1)
; (defglobal ?*select-image*  = -1)
; (defglobal ?*select-obj*  = -1)
; (defglobal ?*select-node-name*  = -1)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; calling comm.clp 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(deffunction translate-meta-class-to-UML-class
          (?from-card ?from-obj ?to-card ?to-obj)

 (printout t translate-meta-class-to-UML-class crlf)

 ;; Translating attributes with the same name 
 (copy-all-field ?from-card ?from-obj ?to-card ?to-obj)

 ;; Translating equivalent attributes (NOT with the same name)
   (instantiate-field ?from-card ?from-obj ?to-card ?to-obj
                      "Object Attributes(real world)"
                      "Object Attribute" )
) ;; deffunction 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; top level function (1) 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; create an UML Class and then copy all 
;; attributes of the selected abstract/concrete class to 
;; this UML class.
;; 
;; invoked by user from custom-menu item on the UML 
;; Class diagram. 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(deffunction meta-to-uml-1 (?card ?notation-type)

   (printout t "----- meta-to-uml-1 -----" crlf)

   ;; check the selected card-type and node type 
   (bind ?card-type (card-get-string-attribute ?*select-card* diagram-type))
   (printout t ?card-type crlf)
   (printout t ?*select-obj* crlf)

   (if (or (and (neq ?card-type            "Meta-Model")
                (neq ?card-type            "Ontology") 
                (neq ?card-type            "Concept Hierarchy") )
           (<   ?*select-obj*         0) 
           (neq   ?*select-obj-type*  "Concrete Class")
           (eq ?*select-node-name*  "")  )
       then (message-box 
             "Error: Please select a concrete class from the Ontology." OK)
            (return -1) )

   (bind ?card-type (card-get-string-attribute ?card diagram-type))

   (if (or (neq ?card-type     "UML Class Diagram")
           (neq ?notation-type "UML Class") )
    then   (return -1)  )

   (bind ?image (node-image-create ?card "Class"))
   (bind ?obj (diagram-image-get-object ?card ?image))
   (diagram-image-move ?card ?image 100 100 )

   ;; translate the meta-model class to a UML Class
   (translate-meta-class-to-UML-class
          ?*select-card* ?*select-obj* ?card ?obj)

   ;; refresh diagram card 
   (my-show-node-name ?card ?obj) 

) ;; deffun 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
