;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; File: quick.clp
;;
;; quick search of a selected node on all cards. 
;;
;; use the Representation in br-racd.clp !! 
;;    (load ./system/quick.clp)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; text slot, button IDs
(defglobal ?*racd-list-box-id* = -1)
(defglobal ?*search-name-id* = -1)
(defglobal ?*show-type-id* = -1)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; call back functions
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; upon called, get the search name from the text
;; slot, and search it within the list-box
;; then reposition the list-box to the selected
;; text 

(deffunction search-button-callbk (?button-id)
  ;; get search name
  (bind ?search-name  (text-get-value ?*search-name-id*))

  ;; find the string in the list-box
  (bind ?pos (list-box-find-string 
              ?*racd-list-box-id* ?search-name))

  ;; roll list-box to that position ?? 
  (list-box-set-selection ?*racd-list-box-id* ?pos)

) ;; deffun 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; upon called, find the item being selected
;; iconize all windows, and open all of the 
;; windows contain that node

(deffunction open-diagram-callbk (?button-id)

  ;; get item = node-name
  (bind ?pos  (list-box-get-selection ?*racd-list-box-id*))
  (bind ?item (list-box-get-string ?*racd-list-box-id* ?pos))
  ;; (printout t "->" ?item "<-" crlf)

  ;; iconize all windows
  (close-all-cards iconize-card)

  ;; open all windows contain that node
  (assert (show-racd-result-windows ?item))  (run)

);; deffun 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; show the found nodes/cards
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defrule show-racd-result-windows
  (show-racd-result-windows ?node-name)
  (racd_card ?card node_info ?type ?node-name ?image ?obj)
=> 
  (card-show ?card)
  (diagram-image-set-shadow-mode ?card ?image 1)
  (my-refresh-diagram ?card)  
  (assert (shadow-node ?card ?image))
) ;; defrule
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defrule get-ride-of-shadow
  (get-ride-of-shadow)
  ?d <- (shadow-node ?card ?image) 
=>
  (diagram-image-set-shadow-mode ?card ?image 0)
  (my-refresh-diagram ?card)  
  (retract ?d)  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defrule shadow-clean-up
  (declare (salience -10000))
  ?d <- (get-ride-of-shadow)
=>   (retract ?d)   )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; delete the shadows that was marked to make the 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(deffunction delete-shadow (?shadow-button)
  (assert   (get-ride-of-shadow))
  (run)
  (return 1)
) ;; deffun 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(deffunction racd-list-box-callbk (?list-box-id)

  (bind ?pos  (list-box-get-selection ?*racd-list-box-id*))
  (bind ?item (list-box-get-string ?*racd-list-box-id* ?pos))
  (printout t "->" ?item "<-" crlf)

  (assert (racd-item-find-type ?item)) (run)
  (return 1) 
);; deffun 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defrule show-selected-item-type
  (racd-item-find-type ?node-name)
  (racd_card ?card node_info ?node-type ?node-name ?image ?obj)
=> 
  (text-set-value ?*show-type-id* ?node-type)
) ;; defrule 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; create a frame for window
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(deffunction create-list-box-for-racd-nodes ()

  (if (or (eq ?*all-racd-node-names* -1)
          (eq ?*all-racd-node-names* (create$)) ) then
      (message-box "NO RACD Nodes Found." OK) 
      (return -1) );; if

  ;;;;;;;;;;; create a frame/window ;;;;;;;;;;;;;;;;;
  (bind ?att-frame (frame-create 0 "All RACD Nodes"
                     -1 -1 500 500))

   ;; the callback fun is stored in att.clp
  (window-add-callback ?att-frame OnClose on-close)
  (window-add-callback ?att-frame OnMenuCommand simple-menu)

  ;;;;;;;;;;;;; Create Menu-buttons and Menu Bar ;;;;;;;;;;;;;;;
  ;;;;; create a menu, ?exit-menu ;;;;; 
  (bind ?exit-menu (menu-create))
  (menu-append ?exit-menu 9 "&Quit")

  (bind ?att-menu-bar (menu-bar-create))

  ;;;;; add Menus to this Menu Bar
  (menu-bar-append ?att-menu-bar ?exit-menu "&Quit")

  ;;;;; add the menu-bar to the frame ;;;;;
  (frame-set-menu-bar ?att-frame ?att-menu-bar)

  ;;;;;;;;;;;; Create a panel on att-frame ;;;;;;;;;;;;;;;;;;

  (bind ?panel (panel-create ?att-frame 0 0 500 500))
  (panel-set-label-position ?panel wxHORIZONTAL)

  ;;;;; create panel items (1) ;;;;;
  (bind ?*search-name-id* (text-create ?panel "" 
                          "Search Exact Name:" "" -1 -1 400))
  (panel-new-line ?panel)

  ;;;;; create panel items (2) ;;;;;
  (bind ?*show-type-id* (text-create ?panel "" 
                         "Show Item Type:" "" -1 -1 400))
  (panel-new-line ?panel)

  ;;;;; create panel items (3) ;;;;;
  (bind ?search-button (button-create ?panel 
                       "search-button-callbk" "Do Search"))

  ;;;;; create panel items (4) ;;;;;
  (bind ?open-button (button-create ?panel 
                      "open-diagram-callbk" "Open Diagram"))

  ;;;;; create panel items (5) ;;;;;
  (bind ?shadow-button (button-create ?panel 
                        "delete-shadow" "Refresh Diagram"))

  ;;;;; create panel items (6) ;;;;;
  (bind ?total-item-id (text-create ?panel "" 
                    "Number of Items:" "" -1 -1 200))
  (panel-new-line ?panel)

  (bind ?value (length$ ?*all-racd-node-names*))
  (text-set-value ?total-item-id (str-cat ?value))

  ;;;;; create panel items (7) ;;;;;
  (bind ?*racd-list-box-id*  (list-box-create ?panel 
        "racd-list-box-callbk"  ""  0  -1 -1 500 400))

  ;; get all list-box-items
  (bind ?item (nth$ 1 ?*all-racd-node-names*))
  (bind ?rest (rest$  ?*all-racd-node-names*))
  (while (neq ?item nil)
         (list-box-append ?*racd-list-box-id* ?item)
         (bind ?item (nth$ 1 ?rest))
         (bind ?rest (rest$  ?rest))
  );; while 

  (panel-new-line ?panel)

  ;;;;;;;;;; fit the content to the window ;;;;;;;;;;;;;
  (window-fit ?panel)
  (window-fit ?att-frame)

  ;;;;;;;;;;;;;; show window ;;;;;;;;;;;;;;;;;;;;;;
  (window-show ?att-frame 1)

) ;; deffun 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; show all racd nodes 
;; 
;;(racd-nodes ?card-title ?node-type ?node-name ?cardID ?image)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defrule show-all-racd-nodes
   (show-all-racd-nodes)
=>   
   (bind ?att-frame (create-list-box-for-racd-nodes))
);; defrule
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defrule show-all-racd-nodes-clear
   (declare (salience -10000))
   (or ?d <- (show-all-racd-nodes)
       ?d <- (show-racd-result-windows ?node-name)
       ?d <- (racd-item-find-type ?node-name))
=> (retract ?d) 
) ;;defrule

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; top function of this file 
;;    
;;    (load "./system/quick.clp")
;;    (show-all-racd-nodes-top)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(deffunction  show-all-racd-nodes-top ()

  (bind ?ans (message-box 
    "Do you want to represent the nodes?" YES-NO))
  (if (eq ?ans YES) then (rep-racd-top) )

  (assert (show-all-racd-nodes)) (run)
  (return 1)
) ;; deffun 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; end of file 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
