;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; File: racdmm.clp
;;
;; translate a model concept from RACD to MM
;; 
;; This function is invoked from a MM card 
;; 
;; ?*select-card* = racd card 
;; ?card = MM card 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; calling comm.clp 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(deffunction translate-racd-to-mm 
         (?from-card ?from-obj ?to-card ?to-obj)

  (printout t translate-racd-to-mm crlf)

 ;; Translating attributes with the same name 
 (copy-all-field ?from-card ?from-obj ?to-card ?to-obj)

 ;; Translating equivalent attributes (NOT with the same name)

) ;; deffun 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Top function 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(deffunction racd-to-mm (?card)

   (printout t "----- racd-to-mm -----" crlf)

   ;; check the selected card-type and node type 
   (bind ?card-type (card-get-string-attribute ?*select-card* diagram-type))
   (printout t ?card-type crlf)
   (printout t ?*select-obj* crlf)

   (if (or (neq ?card-type   "RACD")
           (<   ?*select-obj*     0) )
       then (message-box 
             "Error: Please select a RACD node." OK)
            (return -1) )

   ;; check the created card-type 
   (bind ?card-type (card-get-string-attribute ?card diagram-type))
   (if (neq ?card-type     "Meta-Model")
       then (message-box 
             "Error: This action can only be carried on a Meta-Model" OK)
            (return -1) )
   
   (bind ?ans (message-box "Translate the RACD node to a Concrete class?"
               YES-NO) )
   (if (neq ?ans YES) then 
       (message-box "Abort." OK)
       (return -1) )

   ;; create a concrete class on MM
   (bind ?image (node-image-create ?card "Concrete Class"))
   (bind ?obj   (diagram-image-get-object ?card ?image))
   (diagram-image-move ?card ?image 100 100 )

   ;; translate the RACD to a concrete class
   (translate-racd-to-mm 
         ?*select-card* ?*select-obj* ?card ?obj)

   ;; refresh diagram card ;; calling library.clp 
   (my-show-node-name ?card ?obj) 

   (return 1)
) ;; deffun 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; end of file 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;