;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; File: repall.clp 
;;
;; Author: Jessica Chen-Burger
;; Represent meta-model and BSDM Diagrams
;; 
;;      (load "./system/repall.clp") 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Level 1.1 
;; 
;;    (represent-meta 7994)
;;
;;  rep one diagram of meta-model
;; 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(deffunction  represent-meta   (?card) 

  ;; get image id
  (bind ?image (diagram-card-get-first-node-image ?card))

  (while (neq ?image -1) 

    ;; get node details
    (bind ?obj  (diagram-image-get-object ?card ?image))
    (bind ?type (diagram-object-get-string-attribute ?card ?obj type))
    (bind ?name (diagram-object-get-string-attribute ?card ?obj "Name"))

    ;; rep a meta-model class
    (if (or (eq ?type "Abstract Class") 
            (eq ?type "Concrete Class"))  then 
    (assert (meta_model_class_tmp ?name))  ) ;;if 

    ;; rep an abstract class
    (if (eq ?type "Abstract Class") then 
    (assert (meta_model ?card class ?type ?name ?obj)) ) ;; if 

    ;; rep a concrete class
    (if (eq ?type "Concrete Class") then 
    (assert (meta_model ?card class ?type ?name ?obj))  ) ;; if 

    ;; rep all atts, no need so far 
    ;; (represent-meta-class-atts ?card ?obj)

    ;; get next image id
    (bind ?image (diagram-card-get-next-node-image))
  ) ;; while

  (return 1)  ) ;;deffun 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Level 1.2 
;;
;;    (represent-bsdm  14550)
;; 
;;  rep one diagram of BSDM
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(deffunction  represent-bsdm   (?card) 

  ;; get image id
  (bind ?image (diagram-card-get-first-node-image ?card))

  (while (neq ?image -1) 

    ;; get node details
    (bind ?obj  (diagram-image-get-object ?card ?image))
    (bind ?type (diagram-object-get-string-attribute ?card ?obj type))
    (bind ?name (diagram-object-get-string-attribute ?card ?obj "Name"))

    ;; rep an entity  
    (if (eq ?type "Entity") then 
        (assert (bsdm_card ?card entity ?name ?obj))
    ) ;; if 

    ;; get next node id
    (bind ?image (diagram-card-get-next-node-image))
  );; while 

  (return 1)
) ;; deffun
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Top Function (1) 
;;
;;      (rep-all-models)
;;
;; Rep all diagram cards of meta-model and BSDM
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(deffunction rep-all-models ()

  (bind ?ans (message-box 
        "Do you want to represent all models? (YES)"  YES-NO))
  (if (neq ?ans YES) then 
      (return -1) )

  (unwatch all) ;; to speed up the program 
  (reset)       ;; reset working memory

  ;; get card id
  (bind ?card (hardy-get-first-card))
  (while (neq ?card -1)

      (card-show ?card)

      ;; get card type
      (bind ?card-type (my-card-get-type2 ?card))
 
      ;; (1) represent meta model card
      (if (eq ?card-type "Meta-Model") then 
          (represent-meta ?card)  )

      ;; (2) represent BSDM card
      (if (eq ?card-type "bsdm") then 
          (represent-bsdm ?card)  )

      ;; next card 
      (card-quit ?card)      
      (bind ?card (hardy-get-next-card))
  );; while

  (message-box "Finished Representing All Models." OK)
  (return 1)
) ;; deffun 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Top Function (2) 
;; 
;;  Represent all Meta-Model diagram cards !! 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(deffunction rep-meta-model ()

  (bind ?ans (message-box 
        "Do you want to clear the memory 
and represent all meta-models?"  YES-NO))

  (if (neq ?ans YES) then 
      (return -1) )

  (unwatch all) ;; to speed up the program 
  (reset)       ;; reset working memory

  ;; get card id
  (bind ?card (hardy-get-first-card))
  (while (neq ?card -1)

      ;; get card type
      (bind ?card-type (my-card-get-type2 ?card))
 
      ;; (1) represent meta model card
      (if (eq ?card-type "Meta-Model") then 
          (card-show ?card)
          (represent-meta ?card)  )

      ;; next card 
      (card-quit ?card)      
      (bind ?card (hardy-get-next-card))
  );; while

  (message-box "Finished Representing All Meta-Models." OK)
  (return 1)

) ;; deffun 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; end of file 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
