;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; File:  uml1meta.clp, from UML class model to meta-model
;; 
;;  Author: Jessica Chen-Burger
;;  (load "/hame/jessicac/kbst-em/system/uml1meta.clp")
;; 
;; Function: translate a a UML class to a meta-model class
;; Status:   working fine 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Use the below global variables to rep selected node: 
; 
; (defglobal ?*select-card*  = -1)
; (defglobal ?*select-image*  = -1)
; (defglobal ?*select-obj*  = -1)
; (defglobal ?*select-node-name*  = -1)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; calling comm.clp 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(deffunction translate-UML-class-to-meta-class
          (?from-card ?from-obj ?to-card ?to-obj)

 (printout t translate-meta-class-to-UML-class crlf)

 ;; Translating attributes with the same name 
 (copy-all-field ?from-card ?from-obj ?to-card ?to-obj)

 ;; Translating equivalent attributes (NOT with the same name)
   (instantiate-field ?from-card ?from-obj ?to-card ?to-obj
                      "Object Attribute" 
                      "Object Attributes(real world)" )
) ;; deffunction 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; top level function (1) 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; create an meta-class and then copy all 
;; attributes of the selected UML class to it. 
;; 
;; invoked by user from custom-menu item on the UML 
;; Class diagram. 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(deffunction uml-1-to-meta (?card  ?notation-type)

   (printout t "----- uml1-to-meta -----" crlf)

   ;; check the selected card-type and node type 
   (bind ?card-type (card-get-string-attribute ?*select-card* diagram-type))
   (printout t ?card-type crlf)
   (printout t ?*select-obj* crlf)

   (if (or (neq ?card-type          "UML Class Diagram")
           (<   ?*select-obj*       0) 
           (eq ?*select-node-name*  "") )
       then (message-box 
             "Error: Please select a UML class node." OK)
            (return -1) )

   ;; check the created card-type 
   (bind ?card-type (card-get-string-attribute ?card diagram-type))
   (if (or (neq ?card-type     "Meta-Model")
           (neq ?notation-type "Meta Class") )
       then (message-box 
             "Error:This action can only be carried on a Meta-Model" OK)
            (return -1) )

   ;; translating a model concept

   (bind ?ans (message-box "Translate to an Abstract class?"
               YES-NO) )
   (if (eq ?ans YES) then 
       (bind ?image (node-image-create ?card "Abstract Class")) )

   (if (eq ?ans NO) then 
       (bind ?image (node-image-create ?card "Concrete Class")) )

   ;; create meta-class
   (bind ?obj (diagram-image-get-object ?card ?image))
   (diagram-image-move ?card ?image 100 100 )

   ;; translate the UML Class to a meta-model class object 
   (translate-UML-class-to-meta-class
          ?*select-card* ?*select-obj* ?card ?obj)

   ;; refresh diagram card ;; calling library.clp 
   (my-show-node-name ?card ?obj) 

) ;; deffun 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
